/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include<libglademm.h>
#include<gtkmm.h>
#include<iostream>
#include"prefdialog.h"
#include"statdialog.h"
#include"importexportdlg.h"
#include"engine.h"
#include"pcaloptions.h"
#include"notesengine.h"
#include"dateselect.h"

#if defined(_WIN32) 
// Create the Windows-based definitions
#include<libintl.h>
#endif

class MonthCalendarModel;

class PCalMainWindow : public Gtk::Window
{
    protected:
        Glib::RefPtr<Gnome::Glade::Xml> m_refGlade;
        virtual bool on_delete_event(GdkEventAny* event);
    public:

        // class constructor
        PCalMainWindow(BaseObjectType* cobject, 
                       const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

        // class destructor
        virtual ~PCalMainWindow();

        // the calendar tree view
        Gtk::TreeView* calendarTreeView;
        Gtk::Label* monthLabel;
        Gtk::Label* yearLabel;
        Gtk::Notebook* infoNotebook;
        Gtk::Label* dayInfoLabel;
        Gtk::Label* periodDayNumberLabel;
        Gtk::Label* fertilityLabel;
        Gtk::Label* badFeelingsLabel;
        Gtk::Label* predictionLabel;
        Gtk::Label* statusLabel;
        Gtk::Label* pregnancyWeekLabel;
        Gtk::Label* estimatedBirthLabel;
       
        void start_with_file(Glib::ustring fname); 
        bool select_date_function(const Glib::RefPtr<Gtk::TreeModel>& model,
                                  const Gtk::TreeModel::Path& path, bool);    

        void refreshCalendarView();
        void refreshLegend(); // refresh the legend colors
        void refreshToolbar();

        // the reference to the calendar tree view model_today_date
        // today date
        Glib::Date * model_today_date;
        Glib::Date * model_current_date;
        
        // the month which is currently selected in calendar
        Glib::Date cal_current_date;
        
        // This function opens the help page for the keyword specified
        void show_help(const Glib::ustring keyword = "");
        void show_help_contents();
        
        // the reference to the calendar engine
        PCalEngine * engine;
        
        // The application options class
        PCalOptions * calOptions;
    private:
        // the timer area
        Gtk::Label* timerLabel;
        
        // the references to the main toolbars
        Gtk::Toolbar* mainToolbars[2];
        
        // the reference to the about dialog
        Gtk::Dialog* pAboutDialog;
        // the reference to the preferences dialog
        PrefDialog* pPrefDialog;    
        // the reference to the statistics dialog
        StatDialog* pStatDialog;
        // the reference to the import/export dialog
        IEDialog* pIEDialog;
    
        // the tree popup dialog
        Gtk::Menu* pPopupMenu;
        Gtk::MenuItem* addPopupMenuItem;
        Gtk::MenuItem* deletePopupMenuItem;
        Gtk::MenuItem* addMenstruationMenuItem;
        Gtk::MenuItem* addPregnancyMenuItem;
        Gtk::MenuItem* addBirthMenuItem;
        Gtk::MenuItem* deleteMenstruationMenuItem;

        // File Chooser dialogs
        Gtk::FileChooserDialog* openFileChooser;
        Gtk::HButtonBox* openFileChooserButtonBox;
        Gtk::Button* openFileChooserOKButton;
        Gtk::FileChooserDialog* saveFileChooser;
        Gtk::HButtonBox* saveFileChooserButtonBox;
        Gtk::Button* saveFileChooserOKButton;
        
        // Cache the week days names
        Glib::ustring WDdays[8];
   
        //Signal handlers
        /// The handler for the timer tick. Repaints the current time 
        virtual bool on_timer_tick();
        virtual void on_new_item();
        virtual void on_open_item();
        virtual void on_save_item();
        virtual void on_save_as_item();
        virtual void on_pref_item();
        virtual void on_stat_item();
        virtual void on_ie_item();
        virtual void on_about_item();
        virtual void on_calendar_click(GdkEventButton* event);
        virtual void on_add_menstruation_item();
        virtual void on_add_pregnancy_item();
        virtual void on_add_birth_item();
        virtual void on_delete_menstruation_item();
        virtual void on_jump_to_date_item();
        
        /* This is disabled because current calendar widget do not
           accept focus
        virtual void on_calendar_key_press(GdkEventKey* event); */
        virtual void on_quit_item();

        void nextMonth();
        void nextYear();
        void prevMonth();
        void prevYear();
        void today();

        // this method recalculates the window title
        void updateWindowTitle();
        // if the file is modified the function asks if to save the changes
        // if yes - saves and returns true
        // if no - does nothing and returns true
        // if cancel - does nothing and returns false
        bool checkModified();

        void activateDate(const Glib::Date selDate);
        void refreshStatistics(); // get the latest statistics from the engine
      
        // container for the today day
        int today_year_day;
    
        // current file name
        Glib::ustring current_file;
        
        MonthCalendarModel * calModel;

        // Notes model, view and controls
        NotesModelColumns* notesColumns;
        Glib::RefPtr<Gtk::ListStore> notesModel;
        Gtk::TreeView* notesTreeView;
        Gtk::TextView* notesTextView;
        Gtk::ToolButton* noteAppendButton;
        Gtk::ToolButton* noteRemoveButton;
        Gtk::MenuItem* noteAppendMenuItem;
        Gtk::MenuItem* noteRemoveMenuItem;
        bool select_note_function(
             const Glib::RefPtr<Gtk::TreeModel>& model,
             const Gtk::TreeModel::Path& path,
             bool);
        virtual void on_note_append_item();
        virtual void on_note_remove_item();
        void reload_notes_tree();
        virtual void on_note_text_changed();
        int treeModelDatesCompare(const Gtk::TreeModel::iterator& item1, 
                                  const Gtk::TreeModel::iterator& item2);

        // keyboard selection flag
        bool keyboardSelected;
        
        // activating note flag
        bool activatingNote;
        // resetting note text flag
        bool resettingNoteText;
        
        // this is a date selector object
        TDateSelect dateSelector;
        
        // !!! to be removed in the next version
        Gtk::MenuItem* editMenuitem;
};

#endif // MAINWINDOW_H
